import { Component, h, State, Prop, Element, Watch } from '@stencil/core';
import { store } from '@stencil/redux';
import { addQuestionPool, getFormId, getPartnerEngineConfig, getQuestionSets } from '@boldpenguin/sdk-redux';
import { canGetStore } from '../../utils/store';
export class BpSdkAddQuestionPoolItemContainer {
  constructor() {
    /**
     * Help text
     */
    this.helpText = '';
    /**
     * Add button text
     */
    this.text = '+';
    this.questionSets = [];
    this.useExternal = false;
    this.handleSubscriptions = () => {
      if (canGetStore()) {
        this.storeUnsubscribe && this.storeUnsubscribe();
        store.mapDispatchToProps(this, { addQuestionPool });
        this.storeUnsubscribe = store.mapStateToProps(this, this.mapStateToProps);
      }
    };
    this.mapStateToProps = (state) => {
      return {
        applicationId: getFormId(state),
        config: getPartnerEngineConfig(state),
        questionSets: getQuestionSets(state),
      };
    };
  }
  resetState() { }
  connectedCallback() {
    this.resetState();
    this.handleSubscriptions();
  }
  disconnectedCallback() {
    this.storeUnsubscribe && this.storeUnsubscribe();
  }
  componentWillLoad() {
    this.useExternal = !!customElements.get('bp-sdk-external-add-question-pool-item-container');
    customElements.whenDefined('bp-sdk-external-add-question-pool-item-container').then(() => {
      this.useExternal = true;
    });
    this.handleSubscriptions();
    this.onQuestionSetIdChange(this.questionSetId);
    this.onQuestionIdChange(this.questionId);
  }
  onQuestionSetIdChange(questionSetId) {
    this.updateQuestionPool(questionSetId, this.questionId, this.questionSets);
  }
  onQuestionIdChange(questionId) {
    this.updateQuestionPool(this.questionSetId, questionId, this.questionSets);
  }
  onQuestionSetsChange(questionSets) {
    this.updateQuestionPool(this.questionSetId, this.questionId, questionSets);
  }
  updateQuestionPool(questionSetId, questionId, questionSets) {
    var _a, _b;
    const set = questionSets === null || questionSets === void 0 ? void 0 : questionSets.filter((set) => set.question_set_id === questionSetId)[0];
    const pools = (_b = (_a = set === null || set === void 0 ? void 0 : set.question_set) === null || _a === void 0 ? void 0 : _a.question_pools) !== null && _b !== void 0 ? _b : [];
    this.questionPool = pools.find((pool) => pool.parent_question_id === questionId);
  }
  render() {
    return [this.useExternal && this.renderExternal(), !this.useExternal && this.renderInternal()];
  }
  renderExternal() {
    return (h("bp-sdk-external-add-question-pool-item-container", { applicationId: this.applicationId, config: this.config, helpText: this.helpText, questionId: this.questionId, questionSetId: this.questionSetId, questionSets: this.questionSets, text: this.text }));
  }
  renderInternal() {
    return (h("div", { class: "bp-sdk-add-question-pool-item-container__container" }, this.questionPool && (h("bp-sdk-question-pool-container", { addText: this.questionPool.add_text, code: this.questionPool.code, description: this.questionPool.description, displayText: this.questionPool.display_text, guid: this.questionPool.id, parentQuestionPoolId: this.questionPool.parent_question_pool_id, parentQuestionPoolReferenceId: this.questionPool.parent_question_pool_reference_id, ownerId: this.questionPool.owner_id, parentQuestionId: this.questionPool.parent_question_id, removeText: this.questionPool.remove_text, tenantId: this.questionPool.tenant_id, config: this.config, applicationId: this.applicationId }))));
  }
  static get is() { return "bp-sdk-add-question-pool-item-container"; }
  static get originalStyleUrls() { return {
    "$": ["./bp-sdk-add-question-pool-item-container.css"]
  }; }
  static get styleUrls() { return {
    "$": ["bp-sdk-add-question-pool-item-container.css"]
  }; }
  static get properties() { return {
    "questionId": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Question id"
      },
      "attribute": "question-id",
      "reflect": false
    },
    "helpText": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Help text"
      },
      "attribute": "help-text",
      "reflect": false,
      "defaultValue": "''"
    },
    "questionSetId": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Question set id"
      },
      "attribute": "question-set-id",
      "reflect": false
    },
    "text": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Add button text"
      },
      "attribute": "text",
      "reflect": false,
      "defaultValue": "'+'"
    },
    "applicationId": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "application-id",
      "reflect": false
    },
    "config": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "IPartnerEngineConfig",
        "resolved": "IPartnerEngineConfig",
        "references": {
          "IPartnerEngineConfig": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      }
    },
    "questionSets": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "IApplicationFormQuestionSet[]",
        "resolved": "IApplicationFormQuestionSet[]",
        "references": {
          "IApplicationFormQuestionSet": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "defaultValue": "[]"
    }
  }; }
  static get states() { return {
    "useExternal": {},
    "questionPool": {}
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "questionSetId",
      "methodName": "onQuestionSetIdChange"
    }, {
      "propName": "questionId",
      "methodName": "onQuestionIdChange"
    }, {
      "propName": "questionSets",
      "methodName": "onQuestionSetsChange"
    }]; }
}
