import { QuestionGroupType } from '@boldpenguin/sdk-types';
import { generateId, shouldHideQuestion } from '@boldpenguin/sdk-utility';
import { Component, h, Prop, State } from '@stencil/core';
export class BpSdkQuestionGroup {
  constructor() {
    /**
     * answers in group
     */
    this.answers = [];
    /**
     * Classes for styling group
     */
    this.classes = '';
    /**
     * Label id's for group
     */
    this.labelId = generateId(8);
    this.useExternal = false;
  }
  resetState() { }
  connectedCallback() {
    this.resetState();
  }
  componentWillLoad() {
    this.useExternal = !!customElements.get('bp-sdk-external-question-group');
    customElements.whenDefined('bp-sdk-external-question-group').then(() => {
      this.useExternal = true;
    });
  }
  render() {
    return [this.useExternal && this.renderExternal(), !this.useExternal && this.renderInternal()];
  }
  renderExternal() {
    return (h("bp-sdk-external-question-group", { answers: this.answers, classes: this.classes, label: this.label, labelId: this.labelId, questionGroup: this.questionGroup, type: this.type }));
  }
  shouldRenderLabel() {
    // Adding a check to avoid accordion groups double rendering labels, this can likely be expanded to other group types but I don't have example data to test with
    return this.label && this.type !== QuestionGroupType.Accordion;
  }
  renderInternal() {
    return [
      this.shouldRenderLabel() && h("bp-sdk-question-group-label", { labelId: this.labelId, label: this.label }),
      h("div", { class: this.classes }, this.renderContent()),
    ];
  }
  renderContent() {
    switch (this.type) {
      case QuestionGroupType.Address:
        return h("bp-sdk-address-group", { label: this.label, labelledBy: this.labelId, answers: this.answers });
      case QuestionGroupType.AccumulatePercentage:
        return h("bp-sdk-accumulation-question-group-container", { label: this.label, labelledBy: this.labelId, answers: this.answers });
      case QuestionGroupType.Vehicle:
        return h("bp-sdk-vehicle-question-pool-container", { questionGroup: this.questionGroup });
      case QuestionGroupType.Driver:
        return h("bp-sdk-driver-question-pool-container", { questionGroup: this.questionGroup });
      case QuestionGroupType.Stepper:
        return h("bp-sdk-stepper-group-container", { questionGroup: this.questionGroup });
      case QuestionGroupType.Accordion:
        return h("bp-sdk-accordion", { questionGroup: this.questionGroup });
      default:
        return this.answers.map((answer) => (h("bp-sdk-dynamic-input-redux-container", { answer: answer, shouldHideLabel: shouldHideQuestion(this.label, answer), labelledBy: this.label ? this.labelId : '', id: `bp-sdk-question-group__${answer.question.id}`, class: "bp-sdk-question-group__question-set" })));
    }
  }
  static get is() { return "bp-sdk-question-group"; }
  static get originalStyleUrls() { return {
    "$": ["bp-sdk-question-group.css"]
  }; }
  static get styleUrls() { return {
    "$": ["bp-sdk-question-group.css"]
  }; }
  static get properties() { return {
    "answers": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "IAnswer[]",
        "resolved": "IAnswer[]",
        "references": {
          "IAnswer": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "answers in group"
      },
      "defaultValue": "[]"
    },
    "classes": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Classes for styling group"
      },
      "attribute": "classes",
      "reflect": false,
      "defaultValue": "''"
    },
    "label": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Group label"
      },
      "attribute": "label",
      "reflect": false
    },
    "labelId": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Label id's for group"
      },
      "attribute": "label-id",
      "reflect": false,
      "defaultValue": "generateId(8)"
    },
    "questionGroup": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "IMappedQuestionGroupAnswers",
        "resolved": "IMappedQuestionGroupAnswers",
        "references": {
          "IMappedQuestionGroupAnswers": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Question Group"
      }
    },
    "type": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "group type"
      },
      "attribute": "type",
      "reflect": true
    }
  }; }
  static get states() { return {
    "useExternal": {}
  }; }
}
