import { bindQuoteRequest, quoteActionBind, getApplicationFormCarriersProducts, getApplicationFormState, getCarriersProducts, getCarriersProductsState, getConfigOptions, getPartnerEngineConfig, getQuotesState, timeoutQuoteState, } from '@boldpenguin/sdk-redux';
import { LoadingStates, SdkAnalyticsVariables, } from '@boldpenguin/sdk-types';
import { Component, Element, Event, Fragment, h, Prop, State, Watch } from '@stencil/core';
import { store } from '@stencil/redux';
import isEqual from 'lodash/isEqual';
import { canGetStore } from '../../utils/store';
export class BpSdkQuoteRequestContainer {
  constructor() {
    /**
     * Should show custom logo
     */
    this.showLogo = true;
    /**
     * Should show custom logo
     */
    this.showCustomLogo = false;
    /**
     * Custom logo url
     */
    this.customLogoBaseUrl = '';
    /**
     * Disable send to exchange button being rendered
     */
    this.disableSendToExchange = false;
    /**
     * Available online quotes on application form injected from redux
     */
    this.quotes = [];
    /**
     * Timer in seconds to use for timing out quoting process
     */
    this.quoteTimeout = 90;
    /**
     * Show all quotes irrespective of status.
     */
    this.showAllQuotes = false;
    /**
     * Optional custom function to sort online quotes, default is quotes are sorted by backend
     */
    this.onlineQuotesSortFn = (a, b) => {
      return a.order - b.order;
    };
    this.useExternal = false;
    this.isReady = true;
    this.showQuotePayment = false;
    this.handleSubscriptions = () => {
      if (canGetStore()) {
        this.storeUnsubscribe && this.storeUnsubscribe();
        store.mapDispatchToProps(this, { bindQuoteRequest, timeoutQuoteState, getApplicationFormCarriersProducts, quoteActionBind });
        this.storeUnsubscribe = store.mapStateToProps(this, this.mapStateToProps);
      }
    };
    this.mapStateToProps = (state) => {
      return {
        appFormState: state.application.form.state,
        applicationFormResourceState: getApplicationFormState(state),
        config: getPartnerEngineConfig(state),
        configOptions: getConfigOptions(state),
        carriersProducts: getCarriersProducts(state) || [],
        carriersProductsResourceState: getCarriersProductsState(state),
        quotes: state.application.quotes || [],
        quotesResourceState: getQuotesState(state),
      };
    };
    this.handleQuoteTimeout = () => {
      this.timeoutQuoteState();
    };
    this.handleBind = (quoteId, appFormId, afterResponse) => {
      this.bindQuoteRequest({
        quoteId,
        appFormId,
        afterResponse,
        config: this.config,
      });
    };
    this.handleQuoteActionBind = (action, quote) => {
      // Pay actions have no API to hit and only update internal state logic
      if (action.value === 'pay') {
        this.showQuotePayment = true;
        return;
      }
      this.quoteActionBind({
        action,
        quote,
        config: this.config,
      });
    };
    this.renderQuotes = () => {
      var _a;
      const onlineCarrierIds = this.quotes.map((quote) => quote.carrier.id);
      const filteredOnlineQuotes = this.quotes.filter((quote) => { var _a, _b; return (((_b = (_a = this.configOptions) === null || _a === void 0 ? void 0 : _a.quoteStatusFilter) !== null && _b !== void 0 ? _b : []).includes(quote.request_status) || this.showAllQuotes); });
      if (this.onlineQuotesSortFn) {
        filteredOnlineQuotes.sort(this.onlineQuotesSortFn);
      }
      const filteredOfflineQuotes = this.carriersProducts
        .filter((offlineQuote) => !onlineCarrierIds.includes(offlineQuote.carrier.id))
        .filter((offlineQuote) => !offlineQuote.carrier.online && !offlineQuote.carrier.excess_and_surplus);
      if (this.offlineQuotesSortFn) {
        filteredOfflineQuotes.sort(this.offlineQuotesSortFn);
      }
      return [
        !filteredOnlineQuotes.length && !filteredOfflineQuotes.length && [],
        filteredOnlineQuotes.map((quote) => {
          const quoteInfo = {
            annual_price: quote.annual_price,
            monthly_price: quote.monthly_price,
            number_of_installments: quote.number_of_installments,
            downpayment: quote.downpayment,
            semiannual_price: quote.semiannual_price,
            price_details: quote.price_details,
            quote_number: quote.quote_number,
          };
          const quoteStatus = {
            recommended: quote.recommended,
            selected: quote.selected,
            consumer_selected: quote.consumer_selected,
            focused: quote.focused,
            expired: quote.expired,
          };
          return (h(Fragment, null,
            h("bp-sdk-quote-request", { actions: quote.actions, attachments: quote.attachments, bindable: quote.bindable, carrier: quote.carrier, products: quote.products, quoteInfo: quoteInfo, showLogo: this.showLogo, showCustomLogo: this.showCustomLogo, customLogoBaseUrl: this.customLogoBaseUrl, handleAction: this.handleQuoteActionBind, handleBind: this.handleBind, quoteId: quote.id, appFormId: quote.application_form_id, quoteStatus: quoteStatus, requestStatus: quote.request_status, translatedMessages: quote.translated_messages }),
            this.renderQuotePayment(quote)));
        }),
        ((_a = this.configOptions) === null || _a === void 0 ? void 0 : _a.renderOfflineQuotes) &&
          filteredOfflineQuotes.map((offlineQuote) => {
            return (h("bp-sdk-offline-quote", { carrier: offlineQuote.carrier, product: offlineQuote.product, showLogo: this.showLogo, showCustomLogo: this.showCustomLogo, customLogoBaseUrl: this.customLogoBaseUrl }));
          }),
        !this.disableSendToExchange && h("bp-sdk-send-to-exchange-container", null),
      ];
    };
  }
  resetState() {
    this.showQuotePayment = false;
  }
  connectedCallback() {
    this.resetState();
    this.handleSubscriptions();
  }
  componentWillLoad() {
    this.useExternal = !!customElements.get('bp-sdk-external-quote-request-container');
    customElements.whenDefined('bp-sdk-external-quote-request-container').then(() => {
      this.useExternal = true;
    });
    this.handleSubscriptions();
    this.onApplicationResourceStateChange(this.applicationFormResourceState);
    this.onQuotesResourceStateChange(this.quotesResourceState);
  }
  disconnectedCallback() {
    this.storeUnsubscribe && this.storeUnsubscribe();
  }
  onQuotesChange(quotes, oldQuotes) {
    if (this.useExternal) {
      return;
    }
    if (!isEqual(quotes, oldQuotes)) {
      const label = quotes
        .map((q) => `${q.carrier.name}_${q.annual_price}_${q.bindable ? SdkAnalyticsVariables.additionalQuoteOnline : SdkAnalyticsVariables.additionalQuoteOffline}`)
        .join('; ');
      this.bpAnalyticsTrack.emit({
        eventName: 'Display_Additional',
        eventPayload: {
          label,
          value: 0,
        },
      });
    }
  }
  onApplicationResourceStateChange(newState) {
    this.updateResource(newState, this.quotesResourceState);
  }
  onQuotesResourceStateChange(newState, oldState) {
    this.updateResource(this.applicationFormResourceState, newState);
    if (!!oldState && oldState !== LoadingStates.Loading && newState === LoadingStates.Loading) {
      // Kick off timer to automatically set to quoted after timeout expires
      setTimeout(this.handleQuoteTimeout, this.quoteTimeout * 1000);
    }
  }
  updateResource(applicationForm, quotes) {
    this.isReady = applicationForm === LoadingStates.Loading || quotes !== LoadingStates.Loading || this.configOptions.renderQuotesAsync;
  }
  componentWillUpdate() {
    this.handleSubscriptions();
  }
  render() {
    return [this.useExternal && this.renderExternal(), !this.useExternal && this.renderInternal()];
  }
  renderExternal() {
    return (h("bp-sdk-external-quote-request-container", { appFormState: this.appFormState, applicationFormResourceState: this.applicationFormResourceState, config: this.config, configOptions: this.configOptions, customLogoBaseUrl: this.customLogoBaseUrl, disableSendToExchange: this.disableSendToExchange, eligibilityCarrierProducts: this.carriersProducts, carriersProductsResourceState: this.carriersProductsResourceState, quoteTimeout: this.quoteTimeout, quotes: this.quotes, quotesResourceState: this.quotesResourceState, showCustomLogo: this.showCustomLogo, showLogo: this.showLogo, showAllQuotes: this.showAllQuotes, onlineQuotesSortFn: this.onlineQuotesSortFn, offlineQuotesSortFn: this.offlineQuotesSortFn }));
  }
  renderQuotePayment(quote) {
    var _a;
    if (this.showQuotePayment && ((_a = quote === null || quote === void 0 ? void 0 : quote.actions) === null || _a === void 0 ? void 0 : _a.length) > 0) {
      return h("bp-sdk-quote-payment", { quoteRequest: quote });
    }
  }
  renderInternal() {
    if (!canGetStore()) {
      return null;
    }
    return [
      !this.isReady && h("bp-sdk-quote-load", null),
      this.isReady && ['completed', 'quoted'].includes(this.appFormState) && this.renderQuotes(),
      h("slot", null),
    ];
  }
  static get is() { return "bp-sdk-quote-request-container"; }
  static get properties() { return {
    "showLogo": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Should show custom logo"
      },
      "attribute": "show-logo",
      "reflect": false,
      "defaultValue": "true"
    },
    "showCustomLogo": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Should show custom logo"
      },
      "attribute": "show-custom-logo",
      "reflect": false,
      "defaultValue": "false"
    },
    "customLogoBaseUrl": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Custom logo url"
      },
      "attribute": "custom-logo-base-url",
      "reflect": false,
      "defaultValue": "''"
    },
    "disableSendToExchange": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Disable send to exchange button being rendered"
      },
      "attribute": "disable-send-to-exchange",
      "reflect": false,
      "defaultValue": "false"
    },
    "quotes": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "IQuotesState[]",
        "resolved": "IQuotesState[]",
        "references": {
          "IQuotesState": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Available online quotes on application form injected from redux"
      },
      "defaultValue": "[]"
    },
    "carriersProducts": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "ICarriersProductState[]",
        "resolved": "ICarriersProductState[]",
        "references": {
          "ICarriersProductState": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Available offline quotes injected from redux"
      }
    },
    "applicationFormResourceState": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "LoadingStates",
        "resolved": "LoadingStates.Error | LoadingStates.Loaded | LoadingStates.Loading | LoadingStates.NotLoaded",
        "references": {
          "LoadingStates": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Application form resource state"
      },
      "attribute": "application-form-resource-state",
      "reflect": false
    },
    "quotesResourceState": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "LoadingStates",
        "resolved": "LoadingStates.Error | LoadingStates.Loaded | LoadingStates.Loading | LoadingStates.NotLoaded",
        "references": {
          "LoadingStates": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Available quotes on application form resource state"
      },
      "attribute": "quotes-resource-state",
      "reflect": false
    },
    "appFormState": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "ApplicationFormStates",
        "resolved": "ApplicationFormStates.Completed | ApplicationFormStates.InProgress | ApplicationFormStates.Incomplete | ApplicationFormStates.Quoted",
        "references": {
          "ApplicationFormStates": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Application form state injected from redux"
      },
      "attribute": "app-form-state",
      "reflect": false
    },
    "config": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "IPartnerEngineConfig",
        "resolved": "IPartnerEngineConfig",
        "references": {
          "IPartnerEngineConfig": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "IPartnerEngineConfig injected from redux"
      }
    },
    "carriersProductsResourceState": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "LoadingStates",
        "resolved": "LoadingStates.Error | LoadingStates.Loaded | LoadingStates.Loading | LoadingStates.NotLoaded",
        "references": {
          "LoadingStates": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Offline quotes resources loaded injected from redux"
      },
      "attribute": "carriers-products-resource-state",
      "reflect": false
    },
    "configOptions": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "IConfigOptions",
        "resolved": "IConfigOptions",
        "references": {
          "IConfigOptions": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Web component config options injected from redux"
      }
    },
    "quoteTimeout": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Timer in seconds to use for timing out quoting process"
      },
      "attribute": "quote-timeout",
      "reflect": false,
      "defaultValue": "90"
    },
    "showAllQuotes": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Show all quotes irrespective of status."
      },
      "attribute": "show-all-quotes",
      "reflect": false,
      "defaultValue": "false"
    },
    "onlineQuotesSortFn": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "(a: IQuotesState, b: IQuotesState) => number",
        "resolved": "(a: IQuotesState, b: IQuotesState) => number",
        "references": {
          "IQuotesState": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Optional custom function to sort online quotes, default is quotes are sorted by backend"
      },
      "defaultValue": "(a: IQuotesState, b: IQuotesState) => {\n    return a.order - b.order;\n  }"
    },
    "offlineQuotesSortFn": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "(a: ICarriersProductState, b: ICarriersProductState) => number",
        "resolved": "(a: ICarriersProductState, b: ICarriersProductState) => number",
        "references": {
          "ICarriersProductState": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Optional custom function to sort online quotes, default is quotes are sorted by backend"
      }
    }
  }; }
  static get states() { return {
    "useExternal": {},
    "isReady": {},
    "showQuotePayment": {}
  }; }
  static get events() { return [{
      "method": "bpAnalyticsTrack",
      "name": "bpAnalyticsTrack",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "IAnalyticsTrackEventDetail",
        "resolved": "IAnalyticsTrackEventDetail",
        "references": {
          "IAnalyticsTrackEventDetail": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      }
    }]; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "quotes",
      "methodName": "onQuotesChange"
    }, {
      "propName": "applicationFormResourceState",
      "methodName": "onApplicationResourceStateChange"
    }, {
      "propName": "quotesResourceState",
      "methodName": "onQuotesResourceStateChange"
    }]; }
}
