/**
 * based on https://github.com/SebastianM/angular-google-maps/blob/master/packages/core/services/maps-api-loader/lazy-maps-api-loader.ts
 * license: MIT
 */
export var GoogleMapsScriptProtocol;
(function (GoogleMapsScriptProtocol) {
  GoogleMapsScriptProtocol[GoogleMapsScriptProtocol["HTTP"] = 1] = "HTTP";
  GoogleMapsScriptProtocol[GoogleMapsScriptProtocol["HTTPS"] = 2] = "HTTPS";
  GoogleMapsScriptProtocol[GoogleMapsScriptProtocol["AUTO"] = 3] = "AUTO";
})(GoogleMapsScriptProtocol || (GoogleMapsScriptProtocol = {}));
const win = window;
export class LazyMapsAPILoader {
  constructor() {
    this._SCRIPT_ID = 'bpSdkGoogleMapsApiScript';
    this.callbackName = `bpSdkLazyMapsAPILoader`;
  }
  load() {
    if (window['google'] && win['google'].maps) {
      // Google maps already loaded on the page.
      return Promise.resolve();
    }
    if (this._scriptLoadingPromise) {
      return this._scriptLoadingPromise;
    }
    // this can happen in HMR situations or Stackblitz.io editors.
    const scriptOnPage = document.getElementById(this._SCRIPT_ID);
    if (scriptOnPage) {
      this._assignScriptLoadingPromise(scriptOnPage);
      return this._scriptLoadingPromise;
    }
    const script = document.createElement('script');
    script.type = 'text/javascript';
    script.async = true;
    script.defer = true;
    script.id = this._SCRIPT_ID;
    script.src = this._getScriptSrc(this.callbackName);
    this._assignScriptLoadingPromise(script);
    document.body.appendChild(script);
    return this._scriptLoadingPromise;
  }
  _assignScriptLoadingPromise(scriptElem) {
    this._scriptLoadingPromise = new Promise((resolve, reject) => {
      window[this.callbackName] = () => {
        resolve();
      };
      scriptElem.onerror = (error) => {
        reject(error);
      };
    });
  }
  _getScriptSrc(callbackName) {
    var _a;
    let protocolType = (_a = this.protocol) !== null && _a !== void 0 ? _a : GoogleMapsScriptProtocol.HTTPS;
    let protocol;
    switch (protocolType) {
      case GoogleMapsScriptProtocol.AUTO:
        protocol = '';
        break;
      case GoogleMapsScriptProtocol.HTTP:
        protocol = 'http:';
        break;
      case GoogleMapsScriptProtocol.HTTPS:
        protocol = 'https:';
        break;
    }
    const hostAndPath = this.hostAndPath || 'maps.googleapis.com/maps/api/js';
    const queryParams = {
      v: this.apiVersion || 'quarterly',
      callback: callbackName,
      key: this.apiKey,
      client: this.clientId,
      channel: this.channel,
      libraries: this.libraries,
      region: this.region,
      language: this.language || (this.localeId !== 'en-US' ? this.localeId : null)
    };
    const params = Object.keys(queryParams)
      .filter((k) => queryParams[k] != null)
      .filter((k) => {
      // remove empty arrays
      return (!Array.isArray(queryParams[k]) ||
        (Array.isArray(queryParams[k]) && queryParams[k].length > 0));
    })
      .map((k) => {
      // join arrays as comma seperated strings
      let i = queryParams[k];
      if (Array.isArray(i)) {
        return { key: k, value: i.join(',') };
      }
      return { key: k, value: queryParams[k] };
    })
      .map((entry) => {
      return `${entry.key}=${entry.value}`;
    })
      .join('&');
    return `${protocol}//${hostAndPath}?${params}`;
  }
}
