import { r as registerInstance, c as createEvent, h, F as Fragment, g as getElement } from './index-b4e0290f.js';
import { bindQuoteRequest, timeoutQuoteState, getApplicationFormCarriersProducts, quoteActionBind, getApplicationFormState, getPartnerEngineConfig, getConfigOptions, getCarriersProducts, getCarriersProductsState, getQuotesState } from '@boldpenguin/sdk-redux';
import { f, i } from './index.esm.min-5fa9dcad.js';
import { c as canGetStore, s as store } from './store-6dda1ad2.js';
import { i as isEqual_1 } from './isEqual-ca262489.js';
import './_commonjsHelpers-5e4b1d1a.js';

let BpSdkQuoteRequestContainer = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.bpAnalyticsTrack = createEvent(this, "bpAnalyticsTrack", 7);
    /**
     * Should show custom logo
     */
    this.showLogo = true;
    /**
     * Should show custom logo
     */
    this.showCustomLogo = false;
    /**
     * Custom logo url
     */
    this.customLogoBaseUrl = '';
    /**
     * Disable send to exchange button being rendered
     */
    this.disableSendToExchange = false;
    /**
     * Available online quotes on application form injected from redux
     */
    this.quotes = [];
    /**
     * Timer in seconds to use for timing out quoting process
     */
    this.quoteTimeout = 90;
    /**
     * Show all quotes irrespective of status.
     */
    this.showAllQuotes = false;
    /**
     * Optional custom function to sort online quotes, default is quotes are sorted by backend
     */
    this.onlineQuotesSortFn = (a, b) => {
      return a.order - b.order;
    };
    this.useExternal = false;
    this.isReady = true;
    this.showQuotePayment = false;
    this.handleSubscriptions = () => {
      if (canGetStore()) {
        this.storeUnsubscribe && this.storeUnsubscribe();
        store.mapDispatchToProps(this, { bindQuoteRequest, timeoutQuoteState, getApplicationFormCarriersProducts, quoteActionBind });
        this.storeUnsubscribe = store.mapStateToProps(this, this.mapStateToProps);
      }
    };
    this.mapStateToProps = (state) => {
      return {
        appFormState: state.application.form.state,
        applicationFormResourceState: getApplicationFormState(state),
        config: getPartnerEngineConfig(state),
        configOptions: getConfigOptions(state),
        carriersProducts: getCarriersProducts(state) || [],
        carriersProductsResourceState: getCarriersProductsState(state),
        quotes: state.application.quotes || [],
        quotesResourceState: getQuotesState(state),
      };
    };
    this.handleQuoteTimeout = () => {
      this.timeoutQuoteState();
    };
    this.handleBind = (quoteId, appFormId, afterResponse) => {
      this.bindQuoteRequest({
        quoteId,
        appFormId,
        afterResponse,
        config: this.config,
      });
    };
    this.handleQuoteActionBind = (action, quote) => {
      // Pay actions have no API to hit and only update internal state logic
      if (action.value === 'pay') {
        this.showQuotePayment = true;
        return;
      }
      this.quoteActionBind({
        action,
        quote,
        config: this.config,
      });
    };
    this.renderQuotes = () => {
      var _a;
      const onlineCarrierIds = this.quotes.map((quote) => quote.carrier.id);
      const filteredOnlineQuotes = this.quotes.filter((quote) => { var _a, _b; return (((_b = (_a = this.configOptions) === null || _a === void 0 ? void 0 : _a.quoteStatusFilter) !== null && _b !== void 0 ? _b : []).includes(quote.request_status) || this.showAllQuotes); });
      if (this.onlineQuotesSortFn) {
        filteredOnlineQuotes.sort(this.onlineQuotesSortFn);
      }
      const filteredOfflineQuotes = this.carriersProducts
        .filter((offlineQuote) => !onlineCarrierIds.includes(offlineQuote.carrier.id))
        .filter((offlineQuote) => !offlineQuote.carrier.online && !offlineQuote.carrier.excess_and_surplus);
      if (this.offlineQuotesSortFn) {
        filteredOfflineQuotes.sort(this.offlineQuotesSortFn);
      }
      return [
        !filteredOnlineQuotes.length && !filteredOfflineQuotes.length && [],
        filteredOnlineQuotes.map((quote) => {
          const quoteInfo = {
            annual_price: quote.annual_price,
            monthly_price: quote.monthly_price,
            number_of_installments: quote.number_of_installments,
            downpayment: quote.downpayment,
            semiannual_price: quote.semiannual_price,
            price_details: quote.price_details,
            quote_number: quote.quote_number,
          };
          const quoteStatus = {
            recommended: quote.recommended,
            selected: quote.selected,
            consumer_selected: quote.consumer_selected,
            focused: quote.focused,
            expired: quote.expired,
          };
          return (h(Fragment, null, h("bp-sdk-quote-request", { actions: quote.actions, attachments: quote.attachments, bindable: quote.bindable, carrier: quote.carrier, products: quote.products, quoteInfo: quoteInfo, showLogo: this.showLogo, showCustomLogo: this.showCustomLogo, customLogoBaseUrl: this.customLogoBaseUrl, handleAction: this.handleQuoteActionBind, handleBind: this.handleBind, quoteId: quote.id, appFormId: quote.application_form_id, quoteStatus: quoteStatus, requestStatus: quote.request_status, translatedMessages: quote.translated_messages }), this.renderQuotePayment(quote)));
        }),
        ((_a = this.configOptions) === null || _a === void 0 ? void 0 : _a.renderOfflineQuotes) &&
          filteredOfflineQuotes.map((offlineQuote) => {
            return (h("bp-sdk-offline-quote", { carrier: offlineQuote.carrier, product: offlineQuote.product, showLogo: this.showLogo, showCustomLogo: this.showCustomLogo, customLogoBaseUrl: this.customLogoBaseUrl }));
          }),
        !this.disableSendToExchange && h("bp-sdk-send-to-exchange-container", null),
      ];
    };
  }
  resetState() {
    this.showQuotePayment = false;
  }
  connectedCallback() {
    this.resetState();
    this.handleSubscriptions();
  }
  componentWillLoad() {
    this.useExternal = !!customElements.get('bp-sdk-external-quote-request-container');
    customElements.whenDefined('bp-sdk-external-quote-request-container').then(() => {
      this.useExternal = true;
    });
    this.handleSubscriptions();
    this.onApplicationResourceStateChange(this.applicationFormResourceState);
    this.onQuotesResourceStateChange(this.quotesResourceState);
  }
  disconnectedCallback() {
    this.storeUnsubscribe && this.storeUnsubscribe();
  }
  onQuotesChange(quotes, oldQuotes) {
    if (this.useExternal) {
      return;
    }
    if (!isEqual_1(quotes, oldQuotes)) {
      const label = quotes
        .map((q) => `${q.carrier.name}_${q.annual_price}_${q.bindable ? f.additionalQuoteOnline : f.additionalQuoteOffline}`)
        .join('; ');
      this.bpAnalyticsTrack.emit({
        eventName: 'Display_Additional',
        eventPayload: {
          label,
          value: 0,
        },
      });
    }
  }
  onApplicationResourceStateChange(newState) {
    this.updateResource(newState, this.quotesResourceState);
  }
  onQuotesResourceStateChange(newState, oldState) {
    this.updateResource(this.applicationFormResourceState, newState);
    if (!!oldState && oldState !== i.Loading && newState === i.Loading) {
      // Kick off timer to automatically set to quoted after timeout expires
      setTimeout(this.handleQuoteTimeout, this.quoteTimeout * 1000);
    }
  }
  updateResource(applicationForm, quotes) {
    this.isReady = applicationForm === i.Loading || quotes !== i.Loading || this.configOptions.renderQuotesAsync;
  }
  componentWillUpdate() {
    this.handleSubscriptions();
  }
  render() {
    return [this.useExternal && this.renderExternal(), !this.useExternal && this.renderInternal()];
  }
  renderExternal() {
    return (h("bp-sdk-external-quote-request-container", { appFormState: this.appFormState, applicationFormResourceState: this.applicationFormResourceState, config: this.config, configOptions: this.configOptions, customLogoBaseUrl: this.customLogoBaseUrl, disableSendToExchange: this.disableSendToExchange, eligibilityCarrierProducts: this.carriersProducts, carriersProductsResourceState: this.carriersProductsResourceState, quoteTimeout: this.quoteTimeout, quotes: this.quotes, quotesResourceState: this.quotesResourceState, showCustomLogo: this.showCustomLogo, showLogo: this.showLogo, showAllQuotes: this.showAllQuotes, onlineQuotesSortFn: this.onlineQuotesSortFn, offlineQuotesSortFn: this.offlineQuotesSortFn }));
  }
  renderQuotePayment(quote) {
    var _a;
    if (this.showQuotePayment && ((_a = quote === null || quote === void 0 ? void 0 : quote.actions) === null || _a === void 0 ? void 0 : _a.length) > 0) {
      return h("bp-sdk-quote-payment", { quoteRequest: quote });
    }
  }
  renderInternal() {
    if (!canGetStore()) {
      return null;
    }
    return [
      !this.isReady && h("bp-sdk-quote-load", null),
      this.isReady && ['completed', 'quoted'].includes(this.appFormState) && this.renderQuotes(),
      h("slot", null),
    ];
  }
  get el() { return getElement(this); }
  static get watchers() { return {
    "quotes": ["onQuotesChange"],
    "applicationFormResourceState": ["onApplicationResourceStateChange"],
    "quotesResourceState": ["onQuotesResourceStateChange"]
  }; }
};

export { BpSdkQuoteRequestContainer as bp_sdk_quote_request_container };
